/*******************************************************************************
* File Name: SingleDiode.c
* Version 1.0
*
* Description:
*  This file provides the API source code for single diode temperature measurement. 
*
* Note: none
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#include <device.h>  /* Include all generated header files  */
#include "math.h"    /* Included for using logarithm function */
#include "SingleDiode.h"

/* Diode multiplication factor used for tempertaure calculation  */
volatile float multiply_factor = 0;

/****************************************************************************** 
* Function Name: CalibrateIdac
*******************************************************************************
*
* Summary:
*  Calculates the ratio of IDAC currents used for transistor biasing. The diode
*  multiplication factor is also computed in this function based on the IDAC ratio.
*  
*
* Parameters:  
*  None
*
* Return: 
*  void 
*
* Global variables:
*  Global variable "multiply_factor" is computed in this function 
*
* Reentrance: 
*  No
*******************************************************************************/
void CalibrateIdac()
{
	float CurrentRatio;
    uint8 loop = 0;
    int32 Adc_I1 = 0;   
	int32 Adc_I2 = 0;
	int32 Adc_Offset = 0;
    
	/* Select Vssa input and do ADC offset measurement */
    AMux_Adc_Select(AMUX_ADC_OFFSET_CHANNEL);     
    
	/* Measure offset voltage for specified number of samples. 
	   Averaging of measured voltage is not done since it will get cancelled in the divison operation
	   done while calculating the current ratio */
    for(loop = 0; loop < NUM_OF_SAMPLES_CALIB; loop++)
    {    
        ADC_DelSig_StartConvert();
        ADC_DelSig_IsEndConversion(ADC_DelSig_WAIT_FOR_RESULT);
        Adc_Offset = Adc_Offset +  ADC_DelSig_GetResult32();  
        ADC_DelSig_StopConvert();
    } 
    
	/* Select Calibration resistor voltage channel */
    AMux_Adc_Select(AMUX_ADC_RESISTOR_CHANNEL);	
			
   	IDAC_Diode_SetValue(IDAC_LOW_CURRENT);	/* Bias current I1 */
   	
	/* Measure voltage across calibration resistor for low current (I1)
	   for specified number of samples */
    for(loop = 0; loop < NUM_OF_SAMPLES_CALIB; loop++)
    {		
        ADC_DelSig_StartConvert();
        ADC_DelSig_IsEndConversion(ADC_DelSig_WAIT_FOR_RESULT);
        Adc_I1 = Adc_I1 + ADC_DelSig_GetResult32();  
        ADC_DelSig_StopConvert();
	}		

	IDAC_Diode_SetValue(IDAC_HIGH_CURRENT);	/* Bias current I2 */
	
	/*  Measure voltage across calibration resistor for high current (I2)
	    for specified number of samples   */
	for(loop = 0; loop < NUM_OF_SAMPLES_CALIB; loop++)
    {		
		ADC_DelSig_StartConvert();
        ADC_DelSig_IsEndConversion(ADC_DelSig_WAIT_FOR_RESULT);
        Adc_I2 = Adc_I2 + ADC_DelSig_GetResult32();  
        ADC_DelSig_StopConvert(); 
    }
	
	/* Set IDAC current to zero when not using the IDAC */
	IDAC_Diode_SetValue(0);
	
	/* Subtract the ADC offset voltage from the calibration resistor voltage readings and 
	   calculate the current ratio - I2/I1 */
    Adc_I1 = Adc_I1 - Adc_Offset;
    Adc_I2 = Adc_I2 - Adc_Offset;
    CurrentRatio = (float)Adc_I2 / (float)Adc_I1;
	
	/* Calculate the diode multiplication factor, which is a aglobal variable, for measured IDAC ratio */
    multiply_factor = (Q_BY_K * ADC_VOLTAGE_RESOLUTION)/(IDEALITY_FACTOR * (log(CurrentRatio)));
}

/****************************************************************************** 
* Function Name: InitializeDiodeMeasurement
*******************************************************************************
*
* Summary:
*  Initializes the components required for temperature measurement and also
*  does an initial IDAC calibration
*
* Parameters:  
*  None
*
* Return: 
*  None
*
*
*******************************************************************************/
void  InitializeDiodeMeasurement()
{
    /* Start the components */
    ADC_DelSig_Start();
    IDAC_Diode_Start();
    AMux_Adc_Start(); 
	
	/* Initial IDAC Calibration */
    CalibrateIdac();
}

/****************************************************************************** 
* Function Name: GetDiodeTemperature
*******************************************************************************
*
* Summary:
*  Returns the temperature of the transistor diode based on the Vbe voltage measured
*
* Parameters:  
*  None
*
* Return: 
*  float - diode temperature
*
*
*******************************************************************************/
float GetDiodeTemperature()
{
    uint8 loop;
    int32 Vbe1_count = 0;
	int32 Vbe2_count = 0;
    float Vbe_diff_count;
    float diode_temperature;	
    
	AMux_Adc_Select(AMUX_ADC_DIODE_CHANNEL); /* Select the Base-emitter junction as the ADC input  */
    
	IDAC_Diode_SetValue(IDAC_LOW_CURRENT); /* Bias current I1 */
	
	/* Measure Vbe for I1. Software loop to reduce noise in measurement */
    for(loop = 0; loop < NUM_OF_SAMPLES_VBEDIFF; loop++)
    {
        ADC_DelSig_StartConvert();
        ADC_DelSig_IsEndConversion(ADC_DelSig_WAIT_FOR_RESULT);        
        ADC_DelSig_StopConvert();
        Vbe1_count = Vbe1_count +  ADC_DelSig_GetResult32(); 
	}
        
    IDAC_Diode_SetValue(IDAC_HIGH_CURRENT); /* Bias current I2 */
	
	/* Measure Vbe for I2. Software loop to reduce noise in measurement */
    for(loop = 0; loop < NUM_OF_SAMPLES_VBEDIFF; loop++)
    {
        ADC_DelSig_StartConvert();
        ADC_DelSig_IsEndConversion(ADC_DelSig_WAIT_FOR_RESULT);        
        ADC_DelSig_StopConvert();
        Vbe2_count = Vbe2_count +  ADC_DelSig_GetResult32(); 
	}
	
	IDAC_Diode_SetValue(0); /* Set IDAC current to zero when not using the IDAC */
    
	/* Take the different in Vbe and average it per the software loop cycles above */
    Vbe_diff_count = ((float)(Vbe2_count - Vbe1_count)) / ((float)(NUM_OF_SAMPLES_VBEDIFF));
	
	/* Temperature in degree Centigrade */
	diode_temperature =(Vbe_diff_count * multiply_factor) - ZERO_DEGREE_CELSIUS_KELVIN;
	
    return(diode_temperature);
}



/* [] END OF FILE */
