/*******************************************************************************
* File Name: SingleDiode.h
* Version 1.0
*
* Description:
*  This header file contains the definitions and function prototypes for single 
*  diode temperature measurement.
*
* Note: 
*
********************************************************************************
* Copyright 2008-2010, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/
#if !defined(SINGLEDIODE_H)
#define SINGLEDIODE_H
#include <device.h>  
/********************************************************************************
*   Macro definitions
********************************************************************************/

/********************************************************************************
*
*   Q_BY_K =  (q/k)
*				  
*   Where,
*   	  q - Charge of electron (1.602 x 10^-19 coulomb)
*   	  k - Boltzmann constant (1.38 x 10^-23 joules/kelvin)
*	  
*   Substituing the above values,
*
*   Q_BY_K =   (1.602 * (10^-19)) 
*              __________________________    
*   		   ( 1.38 * (10^-23)) 
*           
*   Q_BY_K = 11608.69
*
*********************************************************************************/
#define Q_BY_K ((float)11608.69)

/* For the 2N3904 transistor that comes with the CY8CKIT-025 kit,
*  the ideality factor of 1.004 is used.
**********************************************************************************/
#define IDEALITY_FACTOR ((float)1.004)

/* Temperature in Kelvin for 0 degree celsius. Used for Kelvin to Degree Celcius conversion of temperature */
#define ZERO_DEGREE_CELSIUS_KELVIN      ((float)273.15)

/***************************** USER ATTENTION REQUIRED ***********************************************
********************* Modify value based on ADC configuration ****************************************
*
* This definition is for the ADC resolution in volts, required for the diode temperature calculation.
* The definition uses the ADC counts per volt definiton from the ADC component header file.
* In this project, only one ADC configuration (CFG1) is used. If there are multiple ADC configurations
* in the project, choose the COUNTS_PER_VOLT definition of the ADC configuration used for diode
* temperature measurement. 
*
*****************************************************************************************************/
#define ADC_VOLTAGE_RESOLUTION (1/(float)ADC_DelSig_CFG1_COUNTS_PER_VOLT)

/***************************** USER ATTENTION REQUIRED ***********************************************
********************* Modify value based on ADC configuration ****************************************
*
* This definition is for the ADC resolution in bits.
* The definition uses the ADC bits resolution definiton from the ADC component header file.
* In this project, only one ADC configuration (CFG1) is used. If there are multiple ADC configurations
* in the project, choose the RESOLUTION definition of the ADC configuration used for diode
* temperature measurement. 
*
* ADC bit resolution is used to determine the number of samples used for averaging in temperature calculation.
* For devices that have 20-bit DS ADC, it is recommended to use the ADC in 16-bits or more configuration.
* For devices that only have a 12-bit DS ADC, it is recommended to use the ADC in maximum possible
* 12-bit mode.
*
*****************************************************************************************************/
#define ADC_BITS_RESOLUTION    ADC_DelSig_CFG1_RESOLUTION

/* Channel number definitions of the Analog multiplexer at ADC input. These are based on TopDesign Schematic connections */
#define AMUX_ADC_RESISTOR_CHANNEL     0
#define AMUX_ADC_DIODE_CHANNEL        1
#define AMUX_ADC_OFFSET_CHANNEL       2

/* IDAC high, low currents used for biasing the transistor diode */
#define IDAC_LOW_CURRENT             10
#define IDAC_HIGH_CURRENT           200

/* Number of ADC samples required for IDAC calibration, diode voltage measurements */

/* Selection of averaging samples based on ADC resolution.
   For devices with only 12-bit ADC, heavy software filtering is required */
#if(ADC_BITS_RESOLUTION <= 12)
	#define NUM_OF_SAMPLES_CALIB            64  
	#define NUM_OF_SAMPLES_VBEDIFF          16 
#else
	#define NUM_OF_SAMPLES_CALIB            4  
	#define NUM_OF_SAMPLES_VBEDIFF          1 
#endif

/* Temperataure change that requires a IDAC ratio recalibration is +/- 10  */
#define CALIB_TEMP_INTERVAL_POS 10
#define CALIB_TEMP_INTERVAL_NEG -10

/********************************************************************************
*   Function Prototypes 
********************************************************************************/
void  InitializeDiodeMeasurement(void);
float GetDiodeTemperature(void);
void  CalibrateIdac(void);

#endif /* SINGLEDIODE_H */		


//[] END OF FILE
